package DE_EPAGES::Core::t::MonitorShared;
use base DE_EPAGES::Core::API::Test;

use strict;
use Test::More tests => 41;
use DE_EPAGES::Core::API::Object::ProgressMonitor;
use DE_EPAGES::Core::API::Error qw( Error HandleError );
use DE_EPAGES::Core::API::File qw( CaptureStream );

sub _NewMonitor {
    my ($FinishSub) = @_;

    my $Monitor = My::Monitor->new(
        ContentSub => sub {
            my (undef, $IsFinished) = @_;
            return $IsFinished ? 'finished' : "# processing\n";
        }
    );
    $Monitor->{'FinishSub'} = $FinishSub if defined $FinishSub;

    return $Monitor;
}

sub testNew {

    my $Monitor = _NewMonitor();
    ok !$Monitor->isStarted, 'new monitor is not started';
    ok !$Monitor->isBlocked, 'new monitor is not blocked';
}

sub testBlockTimeout {

    my $Monitor = _NewMonitor();
    $Monitor->blockTimeOut;

    ok !$Monitor->isStarted, 'blockTimeout: not started';
    ok $Monitor->isBlocked, 'blockTimeout: blocked';

    $Monitor->timeout;
    ok !$Monitor->isStarted, 'blockTimeout: timeout was blocked';
    ok $Monitor->isBlocked, 'blockTimeout: still blocked';
}

sub testTimeout {

    my $Monitor = _NewMonitor();

    my $Output = CaptureStream( *STDOUT, sub { $Monitor->timeout } );
    ok $Monitor->isStarted, 'timeout: monitor was started';
    ok !$Monitor->isBlocked, 'timeout: not blocked';
    is $Output, "# processing\n", 'timeout called writeMonitorContent';
}

sub testContent {

    my $Monitor = _NewMonitor();

    is $Monitor->monitorContent, "# processing\n", 'monitorContent/processing';
    is $Monitor->finishedContent, 'finished', 'monitorContent/finished';

    my $Output = CaptureStream( *STDOUT, sub { $Monitor->writeMonitorContent } );
    is $Output, "# processing\n", 'writeMonitorContent';

    $Monitor->finish; # does nothing
    ok( 1, 'finhish' );
}

sub testRun {

    my $Monitor = _NewMonitor();

    my $TestVar = 0;
    $Monitor->run( sub {
        $TestVar = 1;
        is( $Monitor->isStarted, 0, 'monitor not started (inside)' );
    }, 2 );
    is( $Monitor->isStarted, 0, 'monitor not started (outside)' );
    ok( 1 == $TestVar, 'very fast sub' );

    $Monitor = _NewMonitor();
    $Monitor->run( sub {
        sleep 1;
        $TestVar = 2;
    }, 2);
    ok( !$Monitor->isStarted, 'monitor not started' );
    ok( 2 == $TestVar, 'slow sub' );

    $Monitor = _NewMonitor();
    $Monitor->run( sub {
        $Monitor->timeout;
        $TestVar = 3;
        ok( $Monitor->isStarted, 'monitor started manually (inside)' );
    }, 2);
    ok( $Monitor->isStarted, 'monitor started manually (outside)' );
    ok( 3 == $TestVar, 'manual monitor' );

    $Monitor = _NewMonitor();
    $Monitor->run( sub {
        $Monitor->blockTimeOut;
        sleep 3;
        $TestVar = 4
    }, 2);
    ok( 4 == $TestVar, 'very slow sub, blocked monitor' );
    ok( !$Monitor->isStarted, 'monitor blocked' );
    ok( $Monitor->isBlocked, 'monitor blocked flag set' );

    $Monitor = _NewMonitor();
    $Monitor->run( sub {
        sleep 4;
        $TestVar = 5
    }, 2);
    ok( 5 == $TestVar, 'very slow sub' );
    ok( $Monitor->isStarted, 'monitor started' );
    ok( !$Monitor->isBlocked, 'monitor blocked was reset' );
}

sub testFinish {

    my $FinishCalled = 0;
    my $FinishSub = sub { $FinishCalled = 1; };

    my $Monitor = _NewMonitor( $FinishSub );
    $Monitor->run( sub {}, 1);
    ok( !$FinishCalled, 'finish() was not called, because not started' );

    $FinishCalled = 0;
    $Monitor = _NewMonitor( $FinishSub );
    $Monitor->run( sub { $Monitor->timeout }, 1);
    ok( $FinishCalled, 'finish() was called' );

    $FinishCalled = 0;
    $Monitor = _NewMonitor( $FinishSub );
    eval {
        $Monitor->run( sub {
            $Monitor->timeout;
            Error('TestError');
        }, 1);
        fail 'should not come here, because there was an error';
    };
    HandleError( OnError => sub {
        my ($Error) = @_;
        is $Error->code, 'TestError', 'an error occured';
        return 'FIXED';
    });
    ok( $FinishCalled, 'finish() was called even on error' );
}

sub testProgressMonitor {

    my $Monitor = _NewMonitor();
    my $ProgressMonitor = DE_EPAGES::Core::API::Object::ProgressMonitor->new(
       'ContentTemplate' => "# %4d of %4d products exported.\n",
       'Total'          => 20,
       'Current'        => 0,
       'SharedMonitor'  => $Monitor,
    );

    $Monitor->run( sub {
        $Monitor->timeout;
        for my $i ( 0..4 ) {
            ok( 1, "Item $i" );
            $ProgressMonitor->incCurrent;
#            $ProgressMonitor->write;
            sleep(1);
        }
    }, 2);
    is( $ProgressMonitor->get('Current'), 5, 'work is done' );
}

sub test {
    my $self = shift;
    $self->testNew;
    $self->testContent;
    $self->testBlockTimeout;
    $self->testTimeout;
    $self->testContent;
    $self->testFinish;
    $self->testRun;
    $self->testProgressMonitor;
}

__PACKAGE__->runScript;

package My::Monitor; ## no critic qw(ProhibitMultiplePackages)
use base 'DE_EPAGES::Core::API::Object::MonitorShared';

sub finish {
    my $self = shift;
    $self->{'FinishSub'}->() if $self->{'FinishSub'};
}


